/*
 * UNC001 host-side software. (c) Unicontrollers, Ltd., 2010-2011
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file grab_com.c
 * The implementation of so-called 'chat' mode for uncctl
 * (that 'uncctl -C' thing); actually, the file contains two
 * completely independent implementations, for *nix and WindoZe
 *
 * Written by Andrey V. Stolyarov
 *
 */

#ifndef WIN32

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <stdlib.h>
//#include <stdint.h>

#include <uncusb.h>

const int usleep_timeout = 100000; /* 1/10 sec */

int intercept_com(void *dev_hdl)
{
    int child_pid;
    char buf[128];
    printf("Attaching to the device's serial port... press Ctrl-D to quit\n");
    child_pid = fork();
    if(child_pid == 0) { /* child */
        for(;;) {
            char buf[128];
            int r;
            r = unc001_com_poll(dev_hdl, buf, sizeof(buf));
            if(r < 0) {
                printf("Device connection lost, exiting\n");
                kill(getppid(), SIGTERM);
                exit(1);
            }
            if(r < 1) {
                usleep(usleep_timeout);
            } else {
                int i;
                for(i=0; i<=r; i++) putchar(buf[i]);
            }
        }
    }
    while(fgets(buf, sizeof(buf), stdin)) {
        unc001_display_message(dev_hdl, buf);
    }
    kill(child_pid, SIGKILL);
    waitpid(child_pid, NULL, 0);
    unc001_com_release(dev_hdl);
    return 0;
}

#else   /* Windoze version follows */

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>  /* for Sleep() */

#include "uncusb.h"

const int usleep_timeout = 100000; /* 1/10 sec */

int intercept_com(void *dev_hdl)
{
    char buf[128];
    int bufpos = 0;
    printf("Attaching to the device's serial port... press Ctrl-Z to quit\n");
    for(;;) {
        int i,r;
        char *p;
        r = unc001_com_poll(dev_hdl, buf+bufpos, sizeof(buf)-bufpos);
        if(r < 0) {
            printf("Device connection lost, exiting\n");
            exit(1);
        }
        if(r >= 1) {
            while((p = strchr(buf, '\n'))) {
                *p = 0;
                printf("%s\n", buf);
                memmove(buf, p+1, sizeof(buf)-(p+1-buf));
            } 
            bufpos = strlen(buf);
            if(sizeof(buf)-bufpos < UNCUSB_POLL_BUF_MIN_SIZE) {
                printf("%s", buf);
                buf[0] = 0;
                bufpos = 0;
            }
        }
        i = 0;
        if(feof(stdin))
            return 0;
        while(kbhit()) {
            if(fgets(buf, sizeof(buf), stdin)==NULL) break;
            unc001_display_message(dev_hdl, buf);
        }
        Sleep((usleep_timeout+999) /1000);
    }
}


#endif


