/*
 * UNC001 host-side software. (c) Unicontrollers, Ltd., 2010
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file scanbus.c
 * The main file of the command line tool named scanbus, which
 * scans your USB bus for all connected devices and displays them.
 * Actually this file is not a part of UNC suite, it's more a demo.
 *
 * Written by Andrey V. Stolyarov
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdint.h>
#include <usb.h>


void scan()
{
    struct usb_bus      *bus;
    struct usb_device   *dev;
    char buf[256];
    int rc;

    for(bus=usb_get_busses(); bus; bus=bus->next){
        for(dev=bus->devices; dev; dev=dev->next){
            printf("Device: %x / %x\n",
                   dev->descriptor.idVendor,
                   dev->descriptor.idProduct);
            usb_dev_handle *handle = usb_open(dev);
            if(!handle) {
                fprintf(stderr, "opening usb device %x/%x: %s\n",
                                dev->descriptor.idVendor,
                                dev->descriptor.idProduct,
                                usb_strerror());
                continue;
            }
            if(dev->descriptor.iManufacturer) {
                rc = usb_get_string_simple(handle,
                                   dev->descriptor.iManufacturer,
                                   buf, sizeof(buf));
                if(rc<0) {
                    fprintf(stderr, "querying VendorName for %x/%x: %s\n",
                                    dev->descriptor.idVendor,
                                    dev->descriptor.idProduct,
                                    usb_strerror());
                } else
                if(rc == 0) {
                    printf("    empty VendorName\n");
                } else {
                    printf("    VendorName: %s\n", buf);
                }
            }
            if(dev->descriptor.iProduct) {
                rc = usb_get_string_simple(handle,
                                   dev->descriptor.iProduct,
                                   buf, sizeof(buf));
                if(rc<0) {
                    fprintf(stderr, "querying ProductName for %x/%x: %s\n",
                                    dev->descriptor.idVendor,
                                    dev->descriptor.idProduct,
                                    usb_strerror());
                } else
                if(rc == 0) {
                    printf("    empty ProductName\n");
                } else {
                    printf("    ProductName: %s\n", buf);
                }
            }
            if(dev->descriptor.iSerialNumber) {
                rc = usb_get_string_simple(handle,
                                    dev->descriptor.iSerialNumber,
                                    buf, sizeof(buf));
                if(rc<0) {
                    fprintf(stderr, "querying SerialNum for %x/%x: %s\n",
                                    dev->descriptor.idVendor,
                                    dev->descriptor.idProduct,
                                    usb_strerror());
                    continue;
                }
                if(rc == 0) {
                    printf("    empty SerialNumber\n");
                } else {
                    printf("    SerialNumber: %s\n", buf);
                }
            }
            usb_close(handle);
        }
    }
}

int main()
{
    usb_init();
    usb_find_busses();
    usb_find_devices();
    scan();
    return 0;
}
