/*
 * UNC001 support software. (c) Unicontrollers, Ltd., 2010-2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file _geneeprom.c
 * The program that generates EEPROM image (binary) to be flashed
 * into the device.  EEPROM contains the serial number and tunable
 * options, so the image is generated to contain their default values.
 *
 * Written by Andrey V. Stolyarov
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/* NOTE!
   This program assumes that it is compiled and executed on a
   "little endian" machine, so that the host endiannes is the
   same as the device's.

   However, no checks are performed.  We at Unicontrollers don't
   intend to prepare and program our devices using machines other
   that i386, so we don't need such checks.  If you really wish
   this program to work properly on a "big-endian" machine (such
   as SPARC), then perform some appropriate checks and change the
   code as needed.  The program is not too hard to understand :-)
 */

#define DEFAULT_UNC_ID 0x7775
#define DEFAULT_REACTION_REGISTER 0x60000000
#define DEFAULT_USER_CONFIGURATION 0x00000002       /* 2 * 1 laps */
#define DEFAULT_HARDWARE_CONFIGURATION 0x46000004
                  /* timer ticks after 17920 cycles; inputs at 1st slot */

long gethex(const char *str) 
{
    char *errptr;
    long res = strtol(str, &errptr, 16);
    if(!*str || *errptr) {
        fprintf(stderr, "Invalid hexadecimal number [%s]\n", str);
        exit(3);
    }
    return res;
}


int main(int argc, const char * const * argv)
{
    int fd;
    char image[128];
    int i;

    if(argc != 4) {
        fprintf(stderr, "Usage: %s <file> <version-tag> <factory-serial>\n",
                        argv[0]);
        fprintf(stderr, "   both are specified as hex numbers\n");
        exit(1);
    }

    memset(image, 0xdd, sizeof(image));

    *(uint16_t*)(image+0) = gethex(argv[2]);
    *(uint16_t*)(image+2) = gethex(argv[3]);
    *(uint16_t*)(image+4) = DEFAULT_UNC_ID;

    *(uint32_t*)(image+16) = DEFAULT_HARDWARE_CONFIGURATION;
    *(uint32_t*)(image+20) = DEFAULT_USER_CONFIGURATION;

    for(i=0; i<=7; i++)
        *(uint32_t*)(image+32+4*i) = DEFAULT_REACTION_REGISTER;

    fd = open(argv[1], O_WRONLY|O_CREAT|O_TRUNC, 0666);
    if(fd == -1) {
        perror(argv[1]);
        exit(2);
    }
    write(fd, image, sizeof(image));
    close(fd);

    return 0;
}
