/*
 * UNC001 firmware. (c) Unicontrollers, Ltd., 2010-2011
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file ser_io.c
 * Implements input/output routines for the COM port
 *
 * Written by Andrey V. Stolyarov 
 *
 */


#ifndef F_CPU
#define F_CPU 12000000UL
#endif

#define UART_BAUD_RATE      19200

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "uart.h"

#include "led.h"
#include "atparser.h"

#include "ser_io.h"

#define UART_GRAB_WORKAROUND

#define UART_SOME_ERROR \
    (UART_NO_DATA|UART_FRAME_ERROR|UART_OVERRUN_ERROR|UART_BUFFER_OVERFLOW)

#ifdef UART_GRAB_WORKAROUND
static int16_t input_grabbed = 0;
#else
static uint8_t input_grabbed = 0;
#endif


void init_ser_io(void)
{
    uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU)); 
}

void ser_io_pc(uint8_t c)
{
    if(c == '\n')
        uart_putc('\r');
    uart_putc(c);
}

void grab_uart_input(unsigned char *buf, unsigned char buflen)
{
    unsigned char i;
    uint16_t ch;
#ifdef UART_GRAB_WORKAROUND
    input_grabbed = 5000;
#else
    input_grabbed = 1;
#endif
    for(i=0; i<buflen; i++) {
        ch = uart_getc();
        if(ch & UART_SOME_ERROR) {
           for(; i<buflen; i++) buf[i] = 0;
           break;
        }
        ser_io_pc((buf[i] = (ch == '\r') ? '\n' : ch));
    }
}

void release_uart_input()
{
    input_grabbed = 0;
}


//#include "unc_prim.h"
void process_serial_input()
{
    unsigned int c;
    if(input_grabbed <= 0) {
        while(((c=uart_getc()) & UART_SOME_ERROR) == 0) {
            if(c == '\r') c = '\n';
            ser_io_pc(c);
            //global_state_putchar(c);
#if UNC001_ENABLE_AT_COMMANDS == 1
            atp_handle_char(c);        
#endif
        }
    }
#ifdef UART_GRAB_WORKAROUND
    else
        --input_grabbed;
#endif
}
