/*
 * UNC001 firmware. (c) Unicontrollers, Ltd., 2010-2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file unc.c
 * The main file for the firmware implementation.
 *
 * Written by Andrey V. Stolyarov
 *
 */


#include <avr/wdt.h>
#include <avr/sleep.h> 
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "unc_1w.h"
#include "twowire.h"

#include "usbdrv.h"

#include "ucd.h"
#include "unc_prim.h"
#include "atparser.h"
#include "ser_io.h"
#include "usb_io.h"
#include "print.h"
#include "led.h"

static uint8_t timer_intr_flag = 0;

ISR(TIMER1_COMPA_vect)
{
    TCNT1=0;
    timer_intr_flag = 1;
}

void main() __attribute__ ((noreturn));
void main()
{
    uint8_t i;

    /* init UNC-specific stuff first */
    init_primary_hardware();
    /* init_global_state(); // now done by init_prim_hw */ 
    init_led();
    init_ser_io();

    //wdt_enable(WDTO_1S);
    wdt_disable();  /* disable the watchdog timer for safety */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration,
                               do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        //wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();

    init_timer();
#if UNC001_ENABLE_I2C_SUBSYSTEM > 0
    tw_init();
#endif
    unc1w_init();

    sei();

    for(;;){                /* main event loop */
        //wdt_reset();
        timer_intr_flag = 0;
        sleep_mode();

        usbPoll();
        process_usb_interrupt_input();
        process_serial_input();

        /* the rest is tick-based; skip if there wasn't a tick */
        if(!timer_intr_flag)
            continue;
        ++global_state.timemark;
        led_hook();
#if UNC001_ENABLE_I2C_SUBSYSTEM > 0
        tw_hook();
#endif
        do_input_confirmation();
        process_delayed_actions();
        unc1w_hook();
    }
}






#if 0
        // XXX remove this later
        {        
            static uint8_t cc = 100;
            static uint8_t ch = 'A';
            if(! --cc)  {
                global_state_putchar(ch);
                ch++;
                if(ch>'Z') ch = 'A';
                cc = 100;
            }
        }
        // XXX remove this later
        {
            static uint8_t cc = 100;
            static uint8_t twrflag = 0;
            if(! --cc)  {
                if(++global_state.reserved2[0] % 2 == 0) {
                    tw_send(0x68, "", 1);
                    //tw_recv(0x68, 1);
                    twrflag = 1;
                }
                cc = 100;
            }
            if(twrflag) {
                static uint8_t res[1];
                static uint8_t rc;
                rc = tw_get_received(res);
                if(rc < 0) {
                     global_state.reserved2[1] = -1;
                     twrflag = 0;
                } else if(rc > 0) {
                     global_state.reserved2[1] = res[0] | 0xab00;
                     twrflag = 0;
                }
            }
        }
#endif
