/* 
 * UNC001 firmware. (c) Unicontrollers, Ltd., 2010-2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file usb_io.c
 * Implements the usb io functions called by the usbdrv
 *
 * Written by Andrey V. Stolyarov
 *
 */

#include <avr/eeprom.h>
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

/* now the UNC-specific stuff goes */
#include "unc_prot.h"
#include "unc_prim.h"
#include "ser_io.h"
#include "twowire.h"

#include "usb_io.h"




PROGMEM char usbHidReportDescriptor[78] = {
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)

    0x85, uncreq_query_eeprom,     //   REPORT_ID (uncreq_query_eeprom)
    0x95, 0x7f,                    //   REPORT_COUNT (127)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x80, 0x01,              //   FEATURE (Data,Ary,Abs,Vol,Buf)

    0x85, uncreq_command,          //   REPORT_ID (uncreq_command)
    0x95, 0x07,                    //   REPORT_COUNT (7)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x80, 0x01,              //   FEATURE (Data,Ary,Abs,Vol,Buf)

    0x85, uncreq_long_command,     //   REPORT_ID (uncreq_long_command)
                                   // NB: the same as uncreq_com_poll
    0x95, 0x0f,                    //   REPORT_COUNT (15)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x80, 0x01,              //   FEATURE (Data,Ary,Abs,Vol,Buf)

    0x85, uncreq_zero_query_counters,// REPORT_ID (uncreq_zero_query_counters)
    0x95, 0x10,                    //   REPORT_COUNT (16)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0xa0, 0x01,              //   FEATURE (Data,Ary,Abs,NPrf,Vol,Buf)

    0x85, uncreq_query_fullstate,  //   REPORT_ID (uncreq_query_fullstate)
    0x95, 0x3f,                    //   REPORT_COUNT (63)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x00, 0x01,              //   FEATURE (Data,Ary,Abs,Buf)

    0x85, uncreq_query_configuration,// REPORT_ID (uncreq_query_configuration)
    0x95, 0x3f,                    //   REPORT_COUNT (63)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x00, 0x01,              //   FEATURE (Data,Ary,Abs,Buf)

    0x85, uncreq_query_extrastate, //   REPORT_ID (uncreq_query_extrastate)
    0x95, 0x3f,                    //   REPORT_COUNT (63)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x00, 0x01,              //   FEATURE (Data,Ary,Abs,Buf)

//    0x75, 0x08,                    //   REPORT_SIZE (8)
//    0x95, 0x01,                    //   REPORT_COUNT (1)
//    0x81, 0x06,                    //   INPUT (Data,Var,Rel)
    0xc0                           // END_COLLECTION
};


/* The following variables store the status of the current data transfer */
static uint8_t current_address;
static uint8_t request_buffer[17];  /* we'll never need more */
static uint8_t bytes_remaining;
static uint8_t eeprom_portion = 3;



#if 0
/* XXX remove this later!!! */
#include "print.h"
void measure_eeprom_reading()
{
    uint16_t t2;
    uint16_t i;
    uint16_t addr;
    uint8_t buf[8];
    TCNT1 = 0;

    for(i=0; i<1; i++)
        for(addr = 0; addr<256; addr+=8)
            eeprom_read_block(buf, (char*)addr, 8);

    t2 = TCNT1;
    print_str("EET:");
    print_int(t2);
    print_char('\n');
}
#endif

static void finalize_incoming_request()
{
    uint8_t i;
    switch(request_buffer[1]) {
        case uncreqcmd_set_hwconfig:
            eeprom_write_block(request_buffer+4, (uint8_t *)0+16, 4); 
            /* intentionally no break here */
        case uncreqcmd_restore_from_eeprom:
            init_primary_hardware();
            init_timer();
            break;
        case uncreqcmd_set_userconfig:
            global_configuration.user_config = *(uint32_t*)(request_buffer+4);
            break;
        case uncreqcmd_set_id: /* set device id */
            eeprom_write_block(request_buffer+4, (uint8_t *)0+4, 4); 
            break;
        case uncreqcmd_set_reactions: /* set reaction register */
            global_configuration.reactions[request_buffer[2] & 0x07] =
                *(uint32_t*)(request_buffer+4);
            break;
        case uncreqcmd_save_to_eeprom: /* save to eeprom */
            save_registers_to_eeprom();
            break;
        case uncreqcmd_restore_factory:
            restore_factory_defaults();
            break;
        case uncreqcmd_action: /* perform action */
            perform_action(request_buffer[5], request_buffer[4]);
            break;
        case uncreqcmd_com_release: /* ungrab serial input */
            release_uart_input();
            break;
        case uncreqlcmd_com_message:
            for(i=2; i<16; i++) {
                if(request_buffer[i])
                    ser_io_pc(request_buffer[i]);
            }
            break;
#if UNC001_ENABLE_I2C_SUBSYSTEM > 0
        case uncreqlcmd_bus_command:
            if(request_buffer[2] & 1) {
                if(request_buffer[3] > 7)
                    request_buffer[3] = 7;
                tw_recv(request_buffer[2]>>1, request_buffer[3]);
            } else {
                if(request_buffer[3] > 12)
                    request_buffer[3] = 12;
                tw_send(request_buffer[2]>>1,
                        (char*)request_buffer+4, request_buffer[3]);
            }
            break;
#endif
        case uncreqcmd_clear_msg:
            global_state.buf_start = global_state.buf_end = 0;
            break;
        case uncreqlcmd_write_e_reg:
            if(request_buffer[2] < 48) {
                eeprom_write_block(request_buffer+3,
                    (uint8_t *)128 + request_buffer[2] * 8, 8); 
            }
            break;
    }
}

/* usbFunctionRead() is called when the host requests data */
uint8_t usbFunctionRead(uint8_t *data, uint8_t len)
{
    /* presently we use it ONLY for the query_eeprom request */
    /* actually we ignore the fact that the first byte should match
       the request id; this is not used by anyone anyway */
    if(len > bytes_remaining)
        len = bytes_remaining;
    eeprom_read_block(data,
                      (uint8_t *)((uint16_t)128 * (uint16_t)eeprom_portion) +
                      current_address, len);
    current_address += len;
    bytes_remaining -= len;
    return len;
}

/* usbFunctionWrite() is called when the host sends data */
uint8_t   usbFunctionWrite(uint8_t *data, uint8_t len)
{
    uint8_t i;

    for(i = 0; i<len; i++)
        if(current_address+i < sizeof(request_buffer))
            request_buffer[current_address+i] = data[i];

    current_address += len;
    bytes_remaining -= len;

    if(bytes_remaining == 0)
        finalize_incoming_request();

    return bytes_remaining == 0; /* return 1 if this was the last chunk */
}

/* usbFunctionSetup() called when a new request arrives */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){
                                               /* HID class request */
        request_buffer[0] = rq->wValue.bytes[0];
        usbMsgPtr = request_buffer; /* most often case */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){
            switch(request_buffer[0]) {
                case uncreq_query_eeprom:
                    bytes_remaining = 128;
                    current_address = 0;
                    ++eeprom_portion;
                    eeprom_portion &= 3;
                    return USB_NO_MSG;  /* use usbFunctionWrite() */
                case uncreq_query_fullstate:
                    sync_global_state();
                    usbMsgPtr = (uchar*)(&global_state);
                    return sizeof(global_state);
                case uncreq_query_configuration:
                    eeprom_portion = 3; // it's a useful side effect
                    global_configuration.version = UNC_VERSION;
                    usbMsgPtr = (uchar*)(&global_configuration);
                    return sizeof(global_configuration);
                case uncreq_zero_query_counters:
                    zero_counters((uint16_t*)(request_buffer+1));
                    return sizeof(request_buffer);  /* 8x2 b. counters */
                case uncreq_com_poll:
                    grab_uart_input(request_buffer+1,15);
                    return 16;
                case uncreq_query_extrastate:
                    // sync_global_state(); // no need for this
                    usbMsgPtr = (uchar*)(&extra_state);
                    return sizeof(extra_state);
            }
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            // we only use two requests: 8 and 16 bytes
            bytes_remaining =
                (request_buffer[0] == uncreq_long_command) ? 16 : 8;
            current_address = 0;
            return USB_NO_MSG;  /* use usbFunctionRead()/..Write() */
        }
    }else{
        /* ignore vendor type requests, we don't use any */
    }
    return 0;
}


void process_usb_interrupt_input()
{
    uint8_t c;
    if(!(global_configuration.user_config_bytes[1] & 0x01))
        return;
    if(global_state.buf_end == global_state.buf_start)
        return;
    if(!usbInterruptIsReady())
        return;
    c = (global_state.buf_end > global_state.buf_start) ?
        (global_state.buf_end - global_state.buf_start) :
        (32 - global_state.buf_start);
    if(c > 8) c = 8;
    usbSetInterrupt(global_state.message_buffer + global_state.buf_start, c);
    global_state.buf_start += c;
    global_state.buf_start &= 0x1f;
}
